package com.gic.message.service.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthModeEnum;
import com.gic.auth.dto.UnionEnterpriseResourceDTO;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.binlog.base.entity.GicField;
import com.gic.binlog.base.entity.GicRecord;
import com.gic.binlog.base.entity.enums.GicRecordType;
import com.gic.message.service.MessageHandler;
import com.gic.message.utils.ListToMapUtil;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreAuthorizationApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
/**
 * 
 * @ClassName: StoreMessageHandler
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 15:12
 */
@Component
public class StoreMessageHandler implements MessageHandler {

    public static final String OLD_STATUS = "old_status";
    public static final String NEW_STATUS = "new_status";
    public static final String NEW_OWN_TYPE = "new_own_type";
    public static final Integer NEW_STATUS_VALUE = 2;

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreIndexRefreshHandler storeIndexRefreshHandler;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;

    @Override
    public void handler(ConsumerRecord<String, GicRecord> consumerRecord){
        GicRecord value = consumerRecord.value();
        Map<String, GicField> fieldMap = ListToMapUtil.listToMap(consumerRecord);
        Integer enterpriseId = 0, storeId = 0, storeInfoId;
        if(GicRecordType.INSERT.value() == value.getRecordType().value()){
            enterpriseId = Integer.valueOf(fieldMap.get("enterprise_id").getValue());
            storeId = Integer.valueOf(fieldMap.get("store_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("store_info_id").getValue());
            this.storeIndexRefreshHandler.refreshStoreIndex(enterpriseId, storeId);
            Integer ownType = Integer.valueOf(fieldMap.get("own_type").getValue());
            if(ownType == StoreOwnTypeEnum.OWNER.getCode()){
                this.dealStoreChange(enterpriseId, storeId, storeInfoId, GicRecordType.INSERT.value());
            }
        }
        if(GicRecordType.UPDATE.value() == value.getRecordType().value()) {
            enterpriseId = Integer.valueOf(fieldMap.get("new_enterprise_id").getValue());
            storeId = Integer.valueOf(fieldMap.get("new_store_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("new_store_info_id").getValue());
            if (Integer.valueOf(fieldMap.get(OLD_STATUS).getValue()) == 1 && Integer.valueOf(fieldMap.get(NEW_STATUS).getValue()) == 0) {  //删除
                this.storeApiService.deleteStoreFromEs(enterpriseId, storeId);
            } else if (Integer.valueOf(fieldMap.get(OLD_STATUS).getValue()) == 1 && Integer.valueOf(fieldMap.get(NEW_STATUS).getValue()).equals(NEW_STATUS_VALUE)){  //下线
                //todo 通知会员组（积分成本转移）
            }else {
                this.storeIndexRefreshHandler.refreshStoreIndex(enterpriseId, storeId);
            }
            if (StoreOwnTypeEnum.OWNER.getCode() == Integer.valueOf(fieldMap.get(NEW_OWN_TYPE).getValue())) {
                this.dealStoreChange(enterpriseId, storeId, storeInfoId, GicRecordType.UPDATE.value());
            }
        }
        if(GicRecordType.DELETE.value() == value.getRecordType().value()){
            enterpriseId = Integer.valueOf(fieldMap.get("enterprise_id").getValue());
            storeId = Integer.valueOf(fieldMap.get("store_id").getValue());
            storeInfoId = Integer.valueOf(fieldMap.get("store_info_id").getValue());
            this.storeApiService.deleteStoreFromEs(enterpriseId, storeId);
            Integer ownType = Integer.valueOf(fieldMap.get("own_type").getValue());
            if(ownType == StoreOwnTypeEnum.OWNER.getCode()){
                this.dealStoreChange(enterpriseId, storeId, storeInfoId, GicRecordType.DELETE.value());
            }
        }
    }

    private void dealStoreChange(Integer enterpriseId, Integer storeId, Integer storeInfoId, int recordType){
        ServiceResponse<List<UnionEnterpriseResourceDTO>> resourceResponse = this.unionEnterpriseApiService.listStoreResourceByEnterpriseId(enterpriseId);
        if(CollectionUtils.isNotEmpty(resourceResponse.getResult())){
            for(UnionEnterpriseResourceDTO dto : resourceResponse.getResult()){
                ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService.getStoreWidget(dto.getResource().intValue());
                if(!storeWidget.isSuccess()){
                    continue;
                }
                StoreWidgetDTO storeWidgetDTO = storeWidget.getResult();
                if(storeWidgetDTO != null){
                    if(AuthModeEnum.YES.getCode().equals(storeWidgetDTO.getAuthMode())){
                        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
                        storeSearchDTO.setStoreIds(String.valueOf(storeId));
                        storeSearchDTO.setSearchJson(storeWidgetDTO.getSearchParam());
                        storeSearchDTO.setEnterpriseId(enterpriseId);
                        ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStore(storeSearchDTO, 1, 1);
                        if(!response.isSuccess()){
                            continue;
                        }
                        if(CollectionUtils.isNotEmpty(response.getResult().getResult())){
                            if(GicRecordType.DELETE.value() == recordType){
                                this.storeAuthorizationApiService.cancelStore(storeInfoId, dto.getUnionEnterpriseId());
                            }else {
                                StoreDTO storeDTO = new StoreDTO();
                                storeDTO.setStoreInfoId(storeInfoId);
                                storeDTO.setEnterpriseId(enterpriseId);
                                storeDTO.setFromEnterpriseId(dto.getUnionEnterpriseId());
                                this.storeAuthorizationApiService.saveStore(storeDTO);
                            }
                        }else {
                            this.storeAuthorizationApiService.cancelStore(storeInfoId, dto.getUnionEnterpriseId());
                        }
                    }
                }
            }
        }
    }
}
